@@ -114,11 +114,13 @@ void AddActiveEffectSlots(const ALuint *slotids, size_t count, ALCcontext *conte
         delete curarray;
         curarray = nullptr;
     }
+    std::uninitialized_fill_n(newarray->end(), newcount, nullptr);
 
     curarray = context->mActiveAuxSlots.exchange(newarray, std::memory_order_acq_rel);
     ALCdevice *device{context->mDevice.get()};
     while((device->MixCount.load(std::memory_order_acquire)&1))
         std::this_thread::yield();
+    al::destroy_n(curarray->end(), curarray->size());
     delete curarray;
 }
 
@@ -150,11 +152,13 @@ void RemoveActiveEffectSlots(const ALuint *slotids, size_t count, ALCcontext *co
         delete curarray;
         curarray = nullptr;
     }
+    std::uninitialized_fill_n(newarray->end(), newsize, nullptr);
 
     curarray = context->mActiveAuxSlots.exchange(newarray, std::memory_order_acq_rel);
     ALCdevice *device{context->mDevice.get()};
     while((device->MixCount.load(std::memory_order_acquire)&1))
         std::this_thread::yield();
+    al::destroy_n(curarray->end(), curarray->size());
     delete curarray;
 }
 
