@@ -228,6 +228,8 @@ int OpenSLPlayback::mixerProc()
         PRINTERR(result, "bufferQueue->GetInterface SL_IID_PLAY");
     }
 
+    const size_t frame_step{mDevice->channelsFromFmt()};
+
     std::unique_lock<OpenSLPlayback> dlock{*this};
     if(SL_RESULT_SUCCESS != result)
         aluHandleDisconnect(mDevice, "Failed to get playback buffer: 0x%08x", result);
@@ -263,10 +265,10 @@ int OpenSLPlayback::mixerProc()
 
         auto data = mRing->getWriteVector();
         aluMixData(mDevice, data.first.buf,
-            static_cast<ALuint>(data.first.len*mDevice->UpdateSize));
+            static_cast<ALuint>(data.first.len*mDevice->UpdateSize), frame_step);
         if(data.second.len > 0)
             aluMixData(mDevice, data.second.buf,
-                static_cast<ALuint>(data.second.len*mDevice->UpdateSize));
+                static_cast<ALuint>(data.second.len*mDevice->UpdateSize), frame_step);
 
         size_t todo{data.first.len + data.second.len};
         mRing->writeAdvance(todo);
@@ -521,7 +523,7 @@ bool OpenSLPlayback::reset()
     if(SL_RESULT_SUCCESS == result)
     {
         const ALuint num_updates{mDevice->BufferSize / mDevice->UpdateSize};
-        mRing = CreateRingBuffer(num_updates, mFrameSize*mDevice->UpdateSize, true);
+        mRing = RingBuffer::Create(num_updates, mFrameSize*mDevice->UpdateSize, true);
     }
 
     if(SL_RESULT_SUCCESS != result)
@@ -696,7 +698,7 @@ void OpenSLCapture::open(const ALCchar* name)
             mDevice->Frequency/100*5)};
         ALuint num_updates{(length+update_len-1) / update_len};
 
-        mRing = CreateRingBuffer(num_updates, update_len*mFrameSize, false);
+        mRing = RingBuffer::Create(num_updates, update_len*mFrameSize, false);
 
         mDevice->UpdateSize = update_len;
         mDevice->BufferSize = static_cast<ALuint>(mRing->writeSpace() * update_len);
