@@ -62,9 +62,11 @@ constexpr std::array<float,MAX_AMBI_CHANNELS> AmbiScale::FromN3D;
 constexpr std::array<float,MAX_AMBI_CHANNELS> AmbiScale::FromSN3D;
 constexpr std::array<float,MAX_AMBI_CHANNELS> AmbiScale::FromFuMa;
 constexpr std::array<uint8_t,MAX_AMBI_CHANNELS> AmbiIndex::FromFuMa;
+constexpr std::array<uint8_t,MAX_AMBI2D_CHANNELS> AmbiIndex::FromFuMa2D;
 constexpr std::array<uint8_t,MAX_AMBI_CHANNELS> AmbiIndex::FromACN;
 constexpr std::array<uint8_t,MAX_AMBI2D_CHANNELS> AmbiIndex::From2D;
-constexpr std::array<uint8_t,MAX_AMBI_CHANNELS> AmbiIndex::From3D;
+constexpr std::array<uint8_t,MAX_AMBI_CHANNELS> AmbiIndex::OrderFromChannel;
+constexpr std::array<uint8_t,MAX_AMBI2D_CHANNELS> AmbiIndex::OrderFrom2DChannel;
 
 
 namespace {
@@ -270,8 +272,7 @@ constexpr ChannelMap MonoCfg[1] = {
     { BackRight,   { 2.04124145e-1f, -1.08880247e-1f, -1.88586120e-1f,  1.29099444e-1f,  7.45355993e-2f, -3.73460789e-2f,  0.00000000e+0f } },
 };
 
-void InitNearFieldCtrl(ALCdevice *device, ALfloat ctrl_dist, ALuint order,
-    const al::span<const ALuint,MAX_AMBI_ORDER+1> chans_per_order)
+void InitNearFieldCtrl(ALCdevice *device, ALfloat ctrl_dist, ALuint order, bool is3d)
 {
     /* NFC is only used when AvgSpeakerDist is greater than 0. */
     const char *devname{device->DeviceName.c_str()};
@@ -281,7 +282,12 @@ void InitNearFieldCtrl(ALCdevice *device, ALfloat ctrl_dist, ALuint order,
     device->AvgSpeakerDist = clampf(ctrl_dist, 0.1f, 10.0f);
     TRACE("Using near-field reference distance: %.2f meters\n", device->AvgSpeakerDist);
 
-    auto iter = std::copy(chans_per_order.begin(), chans_per_order.begin()+order+1,
+    static const ALuint chans_per_order2d[MAX_AMBI_ORDER+1]{ 1, 2, 2, 2 };
+    static const ALuint chans_per_order3d[MAX_AMBI_ORDER+1]{ 1, 3, 5, 7 };
+    const al::span<const ALuint> chans_per_order{is3d ? chans_per_order3d : chans_per_order2d,
+        order+1u};
+
+    auto iter = std::copy(chans_per_order.begin(), chans_per_order.end(),
         std::begin(device->NumChannelsPerOrder));
     std::fill(iter, std::end(device->NumChannelsPerOrder), 0u);
 }
@@ -422,16 +428,13 @@ void InitPanning(ALCdevice *device)
 
         ALfloat nfc_delay{ConfigValueFloat(devname, "decoder", "nfc-ref-delay").value_or(0.0f)};
         if(nfc_delay > 0.0f)
-        {
-            static const ALuint chans_per_order[MAX_AMBI_ORDER+1]{ 1, 3, 5, 7 };
             InitNearFieldCtrl(device, nfc_delay * SPEEDOFSOUNDMETRESPERSEC, device->mAmbiOrder,
-                chans_per_order);
-        }
+                true);
     }
     else
     {
         ChannelDec chancoeffs[MAX_OUTPUT_CHANNELS]{};
-        ALuint idxmap[MAX_OUTPUT_CHANNELS]{};
+        ALuint outcount{0};
         for(size_t i{0u};i < chanmap.size();++i)
         {
             const ALuint idx{GetChannelIdxByName(device->RealOut, chanmap[i].ChanName)};
@@ -441,8 +444,8 @@ void InitPanning(ALCdevice *device)
                     GetLabelFromChannel(chanmap[i].ChanName));
                 continue;
             }
-            idxmap[i] = idx;
-            std::copy_n(chanmap[i].Config, coeffcount, chancoeffs[i]);
+            outcount = maxu(outcount, idx+1u);
+            std::copy_n(chanmap[i].Config, coeffcount, chancoeffs[idx]);
         }
 
         /* For non-DevFmtAmbi3D, set the ambisonic order given the mixing
@@ -460,19 +463,15 @@ void InitPanning(ALCdevice *device)
         TRACE("Enabling %s-order%s ambisonic decoder\n",
             (coeffcount > 5) ? "third" :
             (coeffcount > 3) ? "second" : "first",
-            ""
-        );
-        device->AmbiDecoder = al::make_unique<BFormatDec>(coeffcount,
-            static_cast<ALsizei>(chanmap.size()), chancoeffs, idxmap);
+            "");
+        device->AmbiDecoder = BFormatDec::Create(coeffcount,
+            al::span<const ChannelDec>{chancoeffs, outcount});
     }
 }
 
 void InitCustomPanning(ALCdevice *device, bool hqdec, const AmbDecConf *conf,
     const ALuint (&speakermap)[MAX_OUTPUT_CHANNELS])
 {
-    static const ALuint chans_per_order2d[MAX_AMBI_ORDER+1] = { 1, 2, 2, 2 };
-    static const ALuint chans_per_order3d[MAX_AMBI_ORDER+1] = { 1, 3, 5, 7 };
-
     if(!hqdec && conf->FreqBands != 1)
         ERR("Basic renderer uses the high-frequency matrix as single-band (xover_freq = %.0fhz)\n",
             conf->XOverFreq);
@@ -485,7 +484,7 @@ void InitCustomPanning(ALCdevice *device, bool hqdec, const AmbDecConf *conf,
     if((conf->ChanMask&AMBI_PERIPHONIC_MASK))
     {
         count = static_cast<ALuint>(AmbiChannelsFromOrder(order));
-        std::transform(AmbiIndex::From3D.begin(), AmbiIndex::From3D.begin()+count,
+        std::transform(AmbiIndex::FromACN.begin(), AmbiIndex::FromACN.begin()+count,
             std::begin(device->Dry.AmbiMap),
             [](const uint8_t &index) noexcept { return BFChannelConfig{1.0f, index}; }
         );
@@ -506,16 +505,14 @@ void InitCustomPanning(ALCdevice *device, bool hqdec, const AmbDecConf *conf,
         (conf->ChanMask > AMBI_1ORDER_MASK) ? "second" : "first",
         (conf->ChanMask&AMBI_PERIPHONIC_MASK) ? " periphonic" : ""
     );
-    device->AmbiDecoder = al::make_unique<BFormatDec>(conf, hqdec, count, device->Frequency,
-        speakermap);
+    device->AmbiDecoder = BFormatDec::Create(conf, hqdec, count, device->Frequency, speakermap);
 
     auto accum_spkr_dist = std::bind(std::plus<float>{}, _1,
         std::bind(std::mem_fn(&AmbDecConf::SpeakerConf::Distance), _2));
-    const ALfloat avg_dist{
+    const float avg_dist{
         std::accumulate(conf->Speakers.begin(), conf->Speakers.end(), 0.0f, accum_spkr_dist) /
-        static_cast<ALfloat>(conf->Speakers.size())};
-    InitNearFieldCtrl(device, avg_dist, order,
-        (conf->ChanMask&AMBI_PERIPHONIC_MASK) ? chans_per_order3d : chans_per_order2d);
+        static_cast<float>(conf->Speakers.size())};
+    InitNearFieldCtrl(device, avg_dist, order, !!(conf->ChanMask&AMBI_PERIPHONIC_MASK));
 
     InitDistanceComp(device, conf, speakermap);
 }
@@ -523,46 +520,40 @@ void InitCustomPanning(ALCdevice *device, bool hqdec, const AmbDecConf *conf,
 void InitHrtfPanning(ALCdevice *device)
 {
     constexpr float PI{al::MathDefs<float>::Pi()};
-    constexpr float PI_2{al::MathDefs<float>::Pi() / 2.0f};
-    constexpr float PI_4{al::MathDefs<float>::Pi() / 4.0f};
-    constexpr float PI3_4{al::MathDefs<float>::Pi() * 3.0f / 4.0f};
-    const float CornerElev{static_cast<float>(std::atan2(1.0, std::sqrt(2.0)))};
+    constexpr float PI_2{PI / 2.0f};
+    constexpr float PI_4{PI_2 / 2.0f};
+    constexpr float PI3_4{PI_4 * 3.0f};
+    static const float CornerElev{static_cast<float>(std::atan2(1.0, std::sqrt(2.0)))};
     static const AngularPoint AmbiPoints1O[]{
-        { ElevRadius{ CornerElev}, AzimRadius{ -PI_4} },
-        { ElevRadius{ CornerElev}, AzimRadius{-PI3_4} },
-        { ElevRadius{ CornerElev}, AzimRadius{  PI_4} },
-        { ElevRadius{ CornerElev}, AzimRadius{ PI3_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{ -PI_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{-PI3_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{  PI_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{ PI3_4} },
+        { EvRadians{ CornerElev}, AzRadians{ -PI_4} },
+        { EvRadians{ CornerElev}, AzRadians{-PI3_4} },
+        { EvRadians{ CornerElev}, AzRadians{  PI_4} },
+        { EvRadians{ CornerElev}, AzRadians{ PI3_4} },
+        { EvRadians{-CornerElev}, AzRadians{ -PI_4} },
+        { EvRadians{-CornerElev}, AzRadians{-PI3_4} },
+        { EvRadians{-CornerElev}, AzRadians{  PI_4} },
+        { EvRadians{-CornerElev}, AzRadians{ PI3_4} },
     }, AmbiPoints2O[]{
-        { ElevRadius{       0.0f}, AzimRadius{  0.0f} },
-        { ElevRadius{       0.0f}, AzimRadius{    PI} },
-        { ElevRadius{       0.0f}, AzimRadius{ -PI_2} },
-        { ElevRadius{       0.0f}, AzimRadius{  PI_2} },
-        { ElevRadius{       PI_2}, AzimRadius{  0.0f} },
-        { ElevRadius{      -PI_2}, AzimRadius{  0.0f} },
-        { ElevRadius{       PI_4}, AzimRadius{ -PI_2} },
-        { ElevRadius{       PI_4}, AzimRadius{  PI_2} },
-        { ElevRadius{      -PI_4}, AzimRadius{ -PI_2} },
-        { ElevRadius{      -PI_4}, AzimRadius{  PI_2} },
-        { ElevRadius{       PI_4}, AzimRadius{  0.0f} },
-        { ElevRadius{       PI_4}, AzimRadius{    PI} },
-        { ElevRadius{      -PI_4}, AzimRadius{  0.0f} },
-        { ElevRadius{      -PI_4}, AzimRadius{    PI} },
-        { ElevRadius{       0.0f}, AzimRadius{ -PI_4} },
-        { ElevRadius{       0.0f}, AzimRadius{-PI3_4} },
-        { ElevRadius{       0.0f}, AzimRadius{  PI_4} },
-        { ElevRadius{       0.0f}, AzimRadius{ PI3_4} },
-        { ElevRadius{ CornerElev}, AzimRadius{ -PI_4} },
-        { ElevRadius{ CornerElev}, AzimRadius{-PI3_4} },
-        { ElevRadius{ CornerElev}, AzimRadius{  PI_4} },
-        { ElevRadius{ CornerElev}, AzimRadius{ PI3_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{ -PI_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{-PI3_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{  PI_4} },
-        { ElevRadius{-CornerElev}, AzimRadius{ PI3_4} },
+        { EvRadians{      -CornerElev}, AzRadians{            -PI_4} },
+        { EvRadians{      -CornerElev}, AzRadians{           -PI3_4} },
+        { EvRadians{       CornerElev}, AzRadians{           -PI3_4} },
+        { EvRadians{       CornerElev}, AzRadians{            PI3_4} },
+        { EvRadians{       CornerElev}, AzRadians{             PI_4} },
+        { EvRadians{      -CornerElev}, AzRadians{             PI_4} },
+        { EvRadians{      -CornerElev}, AzRadians{            PI3_4} },
+        { EvRadians{       CornerElev}, AzRadians{            -PI_4} },
+        { EvRadians{-1.205932499e+00f}, AzRadians{            -PI_2} },
+        { EvRadians{ 1.205932499e+00f}, AzRadians{             PI_2} },
+        { EvRadians{-1.205932499e+00f}, AzRadians{             PI_2} },
+        { EvRadians{ 1.205932499e+00f}, AzRadians{            -PI_2} },
+        { EvRadians{             0.0f}, AzRadians{-1.205932499e+00f} },
+        { EvRadians{             0.0f}, AzRadians{-1.935660155e+00f} },
+        { EvRadians{             0.0f}, AzRadians{ 1.205932499e+00f} },
+        { EvRadians{             0.0f}, AzRadians{ 1.935660155e+00f} },
+        { EvRadians{-3.648638281e-01f}, AzRadians{               PI} },
+        { EvRadians{ 3.648638281e-01f}, AzRadians{               PI} },
+        { EvRadians{ 3.648638281e-01f}, AzRadians{             0.0f} },
+        { EvRadians{-3.648638281e-01f}, AzRadians{             0.0f} },
     };
     static const float AmbiMatrix1O[][MAX_AMBI_CHANNELS]{
         { 1.250000000e-01f,  1.250000000e-01f,  1.250000000e-01f,  1.250000000e-01f },
@@ -574,39 +565,32 @@ void InitHrtfPanning(ALCdevice *device)
         { 1.250000000e-01f, -1.250000000e-01f, -1.250000000e-01f,  1.250000000e-01f },
         { 1.250000000e-01f, -1.250000000e-01f, -1.250000000e-01f, -1.250000000e-01f },
     }, AmbiMatrix2O[][MAX_AMBI_CHANNELS]{
-        { 3.846153846e-02f,  0.000000000e+00f,  0.000000000e+00f,  6.661733875e-02f,  0.000000000e+00f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f,  8.606629658e-02f },
-        { 3.846153846e-02f,  0.000000000e+00f,  0.000000000e+00f, -6.661733875e-02f,  0.000000000e+00f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f,  8.606629658e-02f },
-        { 3.846153846e-02f,  6.661733875e-02f,  0.000000000e+00f,  0.000000000e+00f,  0.000000000e+00f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f, -8.606629658e-02f },
-        { 3.846153846e-02f, -6.661733875e-02f,  0.000000000e+00f,  0.000000000e+00f,  0.000000000e+00f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f, -8.606629658e-02f },
-        { 3.846153846e-02f,  0.000000000e+00f,  6.661733875e-02f,  0.000000000e+00f,  0.000000000e+00f,  0.000000000e+00f,  9.938079900e-02f,  0.000000000e+00f,  0.000000000e+00f },
-        { 3.846153846e-02f,  0.000000000e+00f, -6.661733875e-02f,  0.000000000e+00f,  0.000000000e+00f,  0.000000000e+00f,  9.938079900e-02f,  0.000000000e+00f,  0.000000000e+00f },
-        { 3.846153846e-02f,  4.710557198e-02f,  4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f,  6.834676493e-02f,  2.484519975e-02f,  0.000000000e+00f, -4.303314829e-02f },
-        { 3.846153846e-02f, -4.710557198e-02f,  4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f, -6.834676493e-02f,  2.484519975e-02f,  0.000000000e+00f, -4.303314829e-02f },
-        { 3.846153846e-02f,  4.710557198e-02f, -4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f, -6.834676493e-02f,  2.484519975e-02f,  0.000000000e+00f, -4.303314829e-02f },
-        { 3.846153846e-02f, -4.710557198e-02f, -4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f,  6.834676493e-02f,  2.484519975e-02f,  0.000000000e+00f, -4.303314829e-02f },
-        { 3.846153846e-02f,  0.000000000e+00f,  4.710557198e-02f,  4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f,  2.484519975e-02f,  6.834676493e-02f,  4.303314829e-02f },
-        { 3.846153846e-02f,  0.000000000e+00f,  4.710557198e-02f, -4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f,  2.484519975e-02f, -6.834676493e-02f,  4.303314829e-02f },
-        { 3.846153846e-02f,  0.000000000e+00f, -4.710557198e-02f,  4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f,  2.484519975e-02f, -6.834676493e-02f,  4.303314829e-02f },
-        { 3.846153846e-02f,  0.000000000e+00f, -4.710557198e-02f, -4.710557198e-02f,  0.000000000e+00f,  0.000000000e+00f,  2.484519975e-02f,  6.834676493e-02f,  4.303314829e-02f },
-        { 3.846153846e-02f,  4.710557198e-02f,  0.000000000e+00f,  4.710557198e-02f,  6.834676493e-02f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f,  0.000000000e+00f },
-        { 3.846153846e-02f,  4.710557198e-02f,  0.000000000e+00f, -4.710557198e-02f, -6.834676493e-02f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f,  0.000000000e+00f },
-        { 3.846153846e-02f, -4.710557198e-02f,  0.000000000e+00f,  4.710557198e-02f, -6.834676493e-02f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f,  0.000000000e+00f },
-        { 3.846153846e-02f, -4.710557198e-02f,  0.000000000e+00f, -4.710557198e-02f,  6.834676493e-02f,  0.000000000e+00f, -4.969039950e-02f,  0.000000000e+00f,  0.000000000e+00f },
-        { 3.846153846e-02f,  3.846153846e-02f,  3.846153846e-02f,  3.846153846e-02f,  4.556450996e-02f,  4.556450996e-02f,  0.000000000e+00f,  4.556450996e-02f,  0.000000000e+00f },
-        { 3.846153846e-02f,  3.846153846e-02f,  3.846153846e-02f, -3.846153846e-02f, -4.556450996e-02f,  4.556450996e-02f,  0.000000000e+00f, -4.556450996e-02f,  0.000000000e+00f },
-        { 3.846153846e-02f, -3.846153846e-02f,  3.846153846e-02f,  3.846153846e-02f, -4.556450996e-02f, -4.556450996e-02f,  0.000000000e+00f,  4.556450996e-02f,  0.000000000e+00f },
-        { 3.846153846e-02f, -3.846153846e-02f,  3.846153846e-02f, -3.846153846e-02f,  4.556450996e-02f, -4.556450996e-02f,  0.000000000e+00f, -4.556450996e-02f,  0.000000000e+00f },
-        { 3.846153846e-02f,  3.846153846e-02f, -3.846153846e-02f,  3.846153846e-02f,  4.556450996e-02f, -4.556450996e-02f,  0.000000000e+00f, -4.556450996e-02f,  0.000000000e+00f },
-        { 3.846153846e-02f,  3.846153846e-02f, -3.846153846e-02f, -3.846153846e-02f, -4.556450996e-02f, -4.556450996e-02f,  0.000000000e+00f,  4.556450996e-02f,  0.000000000e+00f },
-        { 3.846153846e-02f, -3.846153846e-02f, -3.846153846e-02f,  3.846153846e-02f, -4.556450996e-02f,  4.556450996e-02f,  0.000000000e+00f, -4.556450996e-02f,  0.000000000e+00f },
-        { 3.846153846e-02f, -3.846153846e-02f, -3.846153846e-02f, -3.846153846e-02f,  4.556450996e-02f,  4.556450996e-02f,  0.000000000e+00f,  4.556450996e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f,  5.000000000e-02f, -5.000000000e-02f,  5.000000000e-02f,  6.454972244e-02f, -6.454972244e-02f,  0.000000000e+00f, -6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f,  5.000000000e-02f, -5.000000000e-02f, -5.000000000e-02f, -6.454972244e-02f, -6.454972244e-02f,  0.000000000e+00f,  6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f,  5.000000000e-02f,  5.000000000e-02f, -5.000000000e-02f, -6.454972244e-02f,  6.454972244e-02f,  0.000000000e+00f, -6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f, -5.000000000e-02f,  5.000000000e-02f, -5.000000000e-02f,  6.454972244e-02f, -6.454972244e-02f,  0.000000000e+00f, -6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f, -5.000000000e-02f,  5.000000000e-02f,  5.000000000e-02f, -6.454972244e-02f, -6.454972244e-02f,  0.000000000e+00f,  6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f, -5.000000000e-02f, -5.000000000e-02f,  5.000000000e-02f, -6.454972244e-02f,  6.454972244e-02f,  0.000000000e+00f, -6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f, -5.000000000e-02f, -5.000000000e-02f, -5.000000000e-02f,  6.454972244e-02f,  6.454972244e-02f,  0.000000000e+00f,  6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f,  5.000000000e-02f,  5.000000000e-02f,  5.000000000e-02f,  6.454972244e-02f,  6.454972244e-02f,  0.000000000e+00f,  6.454972244e-02f,  0.000000000e+00f },
+        { 5.000000000e-02f,  3.090169944e-02f, -8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f, -6.454972244e-02f,  9.045084972e-02f,  0.000000000e+00f, -1.232790000e-02f },
+        { 5.000000000e-02f, -3.090169944e-02f,  8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f, -6.454972244e-02f,  9.045084972e-02f,  0.000000000e+00f, -1.232790000e-02f },
+        { 5.000000000e-02f, -3.090169944e-02f, -8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f,  6.454972244e-02f,  9.045084972e-02f,  0.000000000e+00f, -1.232790000e-02f },
+        { 5.000000000e-02f,  3.090169944e-02f,  8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f,  6.454972244e-02f,  9.045084972e-02f,  0.000000000e+00f, -1.232790000e-02f },
+        { 5.000000000e-02f,  8.090169944e-02f,  0.000000000e+00f,  3.090169944e-02f,  6.454972244e-02f,  0.000000000e+00f, -5.590169944e-02f,  0.000000000e+00f, -7.216878365e-02f },
+        { 5.000000000e-02f,  8.090169944e-02f,  0.000000000e+00f, -3.090169944e-02f, -6.454972244e-02f,  0.000000000e+00f, -5.590169944e-02f,  0.000000000e+00f, -7.216878365e-02f },
+        { 5.000000000e-02f, -8.090169944e-02f,  0.000000000e+00f,  3.090169944e-02f, -6.454972244e-02f,  0.000000000e+00f, -5.590169944e-02f,  0.000000000e+00f, -7.216878365e-02f },
+        { 5.000000000e-02f, -8.090169944e-02f,  0.000000000e+00f, -3.090169944e-02f,  6.454972244e-02f,  0.000000000e+00f, -5.590169944e-02f,  0.000000000e+00f, -7.216878365e-02f },
+        { 5.000000000e-02f,  0.000000000e+00f, -3.090169944e-02f, -8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f, -3.454915028e-02f,  6.454972244e-02f,  8.449668365e-02f },
+        { 5.000000000e-02f,  0.000000000e+00f,  3.090169944e-02f, -8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f, -3.454915028e-02f, -6.454972244e-02f,  8.449668365e-02f },
+        { 5.000000000e-02f,  0.000000000e+00f,  3.090169944e-02f,  8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f, -3.454915028e-02f,  6.454972244e-02f,  8.449668365e-02f },
+        { 5.000000000e-02f,  0.000000000e+00f, -3.090169944e-02f,  8.090169944e-02f,  0.000000000e+00f,  0.000000000e+00f, -3.454915028e-02f, -6.454972244e-02f,  8.449668365e-02f },
     };
     static const float AmbiOrderHFGain1O[MAX_AMBI_ORDER+1]{
         2.000000000e+00f, 1.154700538e+00f
     }, AmbiOrderHFGain2O[MAX_AMBI_ORDER+1]{
-        2.687419249e+00f, 2.081665999e+00f, 1.074967700e+00f
+        2.357022604e+00f, 1.825741858e+00f, 9.428090416e-01f
     };
-    static const ALuint ChansPerOrder[MAX_AMBI_ORDER+1]{ 1, 3, 5, 7 };
 
     static_assert(al::size(AmbiPoints1O) == al::size(AmbiMatrix1O), "First-Order Ambisonic HRTF mismatch");
     static_assert(al::size(AmbiPoints2O) == al::size(AmbiMatrix2O), "Second-Order Ambisonic HRTF mismatch");
@@ -655,27 +639,21 @@ void InitHrtfPanning(ALCdevice *device)
         (device->mRenderMode == HrtfRender) ? "+ Full " : "",
         device->HrtfName.c_str());
 
-    al::span<const AngularPoint> AmbiPoints{};
-    const float (*AmbiMatrix)[MAX_AMBI_CHANNELS]{};
-    const float *AmbiOrderHFGain{};
+    al::span<const AngularPoint> AmbiPoints{AmbiPoints1O};
+    const float (*AmbiMatrix)[MAX_AMBI_CHANNELS]{AmbiMatrix1O};
+    al::span<const float,MAX_AMBI_ORDER+1> AmbiOrderHFGain{AmbiOrderHFGain1O};
     if(ambi_order >= 2)
     {
         AmbiPoints = AmbiPoints2O;
         AmbiMatrix = AmbiMatrix2O;
         AmbiOrderHFGain = AmbiOrderHFGain2O;
     }
-    else /*if(ambi_order == 1)*/
-    {
-        AmbiPoints = AmbiPoints1O;
-        AmbiMatrix = AmbiMatrix1O;
-        AmbiOrderHFGain = AmbiOrderHFGain1O;
-    }
     device->mAmbiOrder = ambi_order;
 
     const size_t count{AmbiChannelsFromOrder(ambi_order)};
     device->mHrtfState = DirectHrtfState::Create(count);
 
-    std::transform(AmbiIndex::From3D.begin(), AmbiIndex::From3D.begin()+count,
+    std::transform(AmbiIndex::FromACN.begin(), AmbiIndex::FromACN.begin()+count,
         std::begin(device->Dry.AmbiMap),
         [](const uint8_t &index) noexcept { return BFChannelConfig{1.0f, index}; }
     );
@@ -684,8 +662,8 @@ void InitHrtfPanning(ALCdevice *device)
     BuildBFormatHrtf(device->mHrtf, device->mHrtfState.get(), AmbiPoints, AmbiMatrix,
         AmbiOrderHFGain);
 
-    HrtfEntry *Hrtf{device->mHrtf};
-    InitNearFieldCtrl(device, Hrtf->field[0].distance, ambi_order, ChansPerOrder);
+    HrtfStore *Hrtf{device->mHrtf};
+    InitNearFieldCtrl(device, Hrtf->field[0].distance, ambi_order, true);
 }
 
 void InitUhjPanning(ALCdevice *device)
@@ -708,7 +686,7 @@ void InitUhjPanning(ALCdevice *device)
 void aluInitRenderer(ALCdevice *device, ALint hrtf_id, HrtfRequestMode hrtf_appreq, HrtfRequestMode hrtf_userreq)
 {
     /* Hold the HRTF the device last used, in case it's used again. */
-    HrtfEntry *old_hrtf{device->mHrtf};
+    HrtfStore *old_hrtf{device->mHrtf};
 
     device->mHrtfState = nullptr;
     device->mHrtf = nullptr;
@@ -812,30 +790,24 @@ void aluInitRenderer(ALCdevice *device, ALint hrtf_id, HrtfRequestMode hrtf_appr
 
     if(hrtf_id >= 0 && static_cast<ALuint>(hrtf_id) < device->HrtfList.size())
     {
-        const EnumeratedHrtf &entry = device->HrtfList[static_cast<ALuint>(hrtf_id)];
-        HrtfEntry *hrtf{GetLoadedHrtf(entry.hrtf)};
-        if(hrtf && hrtf->sampleRate == device->Frequency)
+        const char *devname{device->DeviceName.c_str()};
+        const std::string &hrtfname = device->HrtfList[static_cast<ALuint>(hrtf_id)];
+        if(HrtfStore *hrtf{GetLoadedHrtf(hrtfname, devname, device->Frequency)})
         {
             device->mHrtf = hrtf;
-            device->HrtfName = entry.name;
+            device->HrtfName = hrtfname;
         }
-        else if(hrtf)
-            hrtf->DecRef();
     }
 
     if(!device->mHrtf)
     {
-        auto find_hrtf = [device](const EnumeratedHrtf &entry) -> bool
+        const char *devname{device->DeviceName.c_str()};
+        auto find_hrtf = [device,devname](const std::string &hrtfname) -> bool
         {
-            HrtfEntry *hrtf{GetLoadedHrtf(entry.hrtf)};
+            HrtfStore *hrtf{GetLoadedHrtf(hrtfname, devname, device->Frequency)};
             if(!hrtf) return false;
-            if(hrtf->sampleRate != device->Frequency)
-            {
-                hrtf->DecRef();
-                return false;
-            }
             device->mHrtf = hrtf;
-            device->HrtfName = entry.name;
+            device->HrtfName = hrtfname;
             return true;
         };
         std::find_if(device->HrtfList.cbegin(), device->HrtfList.cend(), find_hrtf);
@@ -906,8 +878,8 @@ void aluInitEffectPanning(ALeffectslot *slot, ALCdevice *device)
     slot->MixBuffer.resize(count);
     slot->MixBuffer.shrink_to_fit();
 
-    auto acnmap_end = AmbiIndex::From3D.begin() + count;
-    auto iter = std::transform(AmbiIndex::From3D.begin(), acnmap_end, slot->Wet.AmbiMap.begin(),
+    auto acnmap_end = AmbiIndex::FromACN.begin() + count;
+    auto iter = std::transform(AmbiIndex::FromACN.begin(), acnmap_end, slot->Wet.AmbiMap.begin(),
         [](const uint8_t &acn) noexcept -> BFChannelConfig
         { return BFChannelConfig{1.0f, acn}; }
     );
@@ -926,19 +898,20 @@ void CalcAmbiCoeffs(const float y, const float z, const float x, const float spr
     coeffs[2]  = 1.732050808f * z; /* ACN 2 = sqrt(3) * Z */
     coeffs[3]  = 1.732050808f * x; /* ACN 3 = sqrt(3) * X */
     /* Second-order */
-    coeffs[4]  = 3.872983346f * x * y;             /* ACN 4 = sqrt(15) * X * Y */
-    coeffs[5]  = 3.872983346f * y * z;             /* ACN 5 = sqrt(15) * Y * Z */
-    coeffs[6]  = 1.118033989f * (z*z*3.0f - 1.0f); /* ACN 6 = sqrt(5)/2 * (3*Z*Z - 1) */
-    coeffs[7]  = 3.872983346f * x * z;             /* ACN 7 = sqrt(15) * X * Z */
-    coeffs[8]  = 1.936491673f * (x*x - y*y);       /* ACN 8 = sqrt(15)/2 * (X*X - Y*Y) */
+    const float xx{x*x}, yy{y*y}, zz{z*z};
+    coeffs[4]  = 3.872983346f * x * y;            /* ACN 4 = sqrt(15) * X * Y */
+    coeffs[5]  = 3.872983346f * y * z;            /* ACN 5 = sqrt(15) * Y * Z */
+    coeffs[6]  = 1.118033989f * (3.0f*zz - 1.0f); /* ACN 6 = sqrt(5)/2 * (3*Z*Z - 1) */
+    coeffs[7]  = 3.872983346f * x * z;            /* ACN 7 = sqrt(15) * X * Z */
+    coeffs[8]  = 1.936491673f * (xx - yy);        /* ACN 8 = sqrt(15)/2 * (X*X - Y*Y) */
     /* Third-order */
-    coeffs[9]  =  2.091650066f * y * (x*x*3.0f - y*y);  /* ACN  9 = sqrt(35/8) * Y * (3*X*X - Y*Y) */
-    coeffs[10] = 10.246950766f * z * x * y;             /* ACN 10 = sqrt(105) * Z * X * Y */
-    coeffs[11] =  1.620185175f * y * (z*z*5.0f - 1.0f); /* ACN 11 = sqrt(21/8) * Y * (5*Z*Z - 1) */
-    coeffs[12] =  1.322875656f * z * (z*z*5.0f - 3.0f); /* ACN 12 = sqrt(7)/2 * Z * (5*Z*Z - 3) */
-    coeffs[13] =  1.620185175f * x * (z*z*5.0f - 1.0f); /* ACN 13 = sqrt(21/8) * X * (5*Z*Z - 1) */
-    coeffs[14] =  5.123475383f * z * (x*x - y*y);       /* ACN 14 = sqrt(105)/2 * Z * (X*X - Y*Y) */
-    coeffs[15] =  2.091650066f * x * (x*x - y*y*3.0f);  /* ACN 15 = sqrt(35/8) * X * (X*X - 3*Y*Y) */
+    coeffs[9]  =  2.091650066f * y * (3.0f*xx - yy);   /* ACN  9 = sqrt(35/8) * Y * (3*X*X - Y*Y) */
+    coeffs[10] = 10.246950766f * z * x * y;            /* ACN 10 = sqrt(105) * Z * X * Y */
+    coeffs[11] =  1.620185175f * y * (5.0f*zz - 1.0f); /* ACN 11 = sqrt(21/8) * Y * (5*Z*Z - 1) */
+    coeffs[12] =  1.322875656f * z * (5.0f*zz - 3.0f); /* ACN 12 = sqrt(7)/2 * Z * (5*Z*Z - 3) */
+    coeffs[13] =  1.620185175f * x * (5.0f*zz - 1.0f); /* ACN 13 = sqrt(21/8) * X * (5*Z*Z - 1) */
+    coeffs[14] =  5.123475383f * z * (xx - yy);        /* ACN 14 = sqrt(105)/2 * Z * (X*X - Y*Y) */
+    coeffs[15] =  2.091650066f * x * (xx - 3.0f*yy);   /* ACN 15 = sqrt(35/8) * X * (X*X - 3*Y*Y) */
     /* Fourth-order */
     /* ACN 16 = sqrt(35)*3/2 * X * Y * (X*X - Y*Y) */
     /* ACN 17 = sqrt(35/2)*3/2 * (3*X*X - Y*Y) * Y * Z */
